#include "epgsearchext.h"
#include <vdr/tools.h>
#include "menu_searchresults.h"
#include <ctype.h>
#include <math.h>

char *SearchModes[3];
char *DaysOfWeek[8];

int     q_manual_update; // data has change --> question for manual update
cSearchExts SearchExts;

void ToLower(char* szText)
{
	if (!szText)
		return;
	for (int loop = 0; szText[loop] !=0; loop++)
        szText[loop] = tolower(szText[loop]);
}

// -- cSearchExt -----------------------------------------------------------------

char *cSearchExt::buffer = NULL;

cSearchExt::cSearchExt(void)
{
  *search = 0;
  options = 1;
  useTime = false;
  startTime = 0000;
  stopTime = 2359;
  useChannel = false;
  channelMin = Channels.GetByNumber(cDevice::CurrentChannel());
  channelMax = Channels.GetByNumber(cDevice::CurrentChannel());
  useCase = false;
  mode = 0;
  useTitle = true;
  useSubtitle = true;
  useDescription = true;
  useDuration = false;
  useAsSearchTimer = false;
  useDayOfWeek = false;
  DayOfWeek = 0;
  buffer = NULL;
  *directory = 0;
  useEpisode = 0;
}

cSearchExt::~cSearchExt(void)
{
  if (buffer) {
    free(buffer);
    buffer = NULL;
  }
}

cSearchExt& cSearchExt::operator= (const cSearchExt &SearchExt)
{
  memcpy(this, &SearchExt, sizeof(*this));
  return *this;
}

bool cSearchExt::operator< (const cListObject &ListObject)
{
  cSearchExt *SE = (cSearchExt *)&ListObject;
  return strcasecmp(search, SE->search) < 0;
}

const char *cSearchExt::ToText(void)
{
  char tmp_Start[5] = "";
  char tmp_Stop[5] = "";
  char tmp_minDuration[5] = "";
  char tmp_maxDuration[5] = "";
  char tmp_chanSel[10] = "";

  free(buffer);
  strreplace(search, ':', '|');
  strreplace(directory, ':', '|');

  if (useTime)
  {
    sprintf(tmp_Start, "%04d", startTime);
    sprintf(tmp_Stop, "%04d", stopTime);
  }
  if (useDuration)
  {
    sprintf(tmp_minDuration, "%04d", minDuration);
    sprintf(tmp_maxDuration, "%04d", maxDuration);
  }

  if (useChannel)
  {
      if (channelMin->Number() < channelMax->Number())
	  sprintf(tmp_chanSel, "%d-%d", channelMin->Number(), channelMax->Number());
      else
	  sprintf(tmp_chanSel, "%d", channelMin->Number());
  }

  asprintf(&buffer, "%s:%d:%s:%s:%d:%s:%d:%d:%d:%d:%d:%d:%s:%s:%d:%d:%d:%d:%s\n",
	   search,
	   useTime,
	   tmp_Start,
	   tmp_Stop,
	   useChannel,
	   useChannel?tmp_chanSel:"0",
	   useCase,
	   mode,
	   useTitle,
	   useSubtitle,
	   useDescription,
	   useDuration,
	   tmp_minDuration,
	   tmp_maxDuration,
	   useAsSearchTimer,
	   useDayOfWeek,
	   DayOfWeek,
	   useEpisode,
	   directory);


  strreplace(search, '|', ':');
  strreplace(directory, '|', ':');


  return buffer;
}

bool cSearchExt::Parse(const char *s)
{
  char *line;
  char *pos;
  char *pos_next;
  int parameter = 1;
  int valuelen;
  char value[MaxFileName];

  pos = line = strdup(s);
  pos_next = pos + strlen(pos);
  if (*pos_next == '\n') *pos_next = 0;
  while (*pos) {
    while (*pos == ' ') pos++;
    if (*pos) {
      if (*pos != ':') {
        pos_next = strchr(pos, ':');
        if (!pos_next)
          pos_next = pos + strlen(pos);
        valuelen = pos_next - pos + 1;
        if (valuelen > MaxFileName) valuelen = MaxFileName;
        strn0cpy(value, pos, valuelen);
        pos = pos_next;
        switch (parameter) {
	    case 1:  strcpy(search, value);
	      break;
	    case 2:  useTime = atoi(value);
	      break;
	    case 3:  startTime = atoi(value);
		break;
	    case 4:  stopTime = atoi(value);
	      break;
	    case 5:  useChannel = atoi(value);
		break;
	    case 6:  
		if (!useChannel)
		{
		    channelMin = NULL;
		    channelMax = NULL;
		}
		else
		{
		    int minNum=0, maxNum=0;
		    int fields = sscanf(value, "%d-%d", &minNum, &maxNum);
		    if (fields == 1)
			maxNum = minNum;
		    channelMin = Channels.GetByNumber(minNum);
		    channelMax = Channels.GetByNumber(maxNum);
		    if ((!channelMin && minNum>0) ||
			(!channelMax && maxNum>0))
		    {
			free(line);
			return false;
		    }
		}
	      break;
	    case 7:  useCase = atoi(value);
	      break;
	    case 8:  mode = atoi(value);
		break;
	    case 9: useTitle = atoi(value);
		break;
	    case 10: useSubtitle = atoi(value);
		break;
	    case 11: useDescription = atoi(value);
		break;
	    case 12: useDuration = atoi(value);
		break;
	    case 13: minDuration = atoi(value);
		break;
	    case 14: maxDuration = atoi(value);
		break;
	    case 15: useAsSearchTimer = atoi(value);
			break;
	    case 16: useDayOfWeek = atoi(value);
			break;
	    case 17: DayOfWeek = atoi(value);
			break;
	    case 18: useEpisode = atoi(value);
			break;
	    case 19:  strcpy(directory, value);
	      break;
        } //switch
      }
      parameter++;
    }
    if (*pos) pos++;
  } //while

  strreplace(search, '|', ':');

  free(line);
  return (parameter >= 11) ? true : false;
}

bool cSearchExt::Save(FILE *f)
{
  return fprintf(f, ToText()) > 0;
}

cEvent * cSearchExt::GetEventBySearchExt(const cSchedule *schedules, const cEvent *Start) 
{
  cEvent *pe = NULL;
  cEvent *p1=schedules->Events()->First();
  // first we have to search for the Start item
  if(Start) {
     cEvent *p;
     for (p = schedules->Events()->First(); p && p!=Start; p = schedules->Events()->Next(p));
     if(p)
        p1=schedules->Events()->Next(p);
     }

  time_t tNow=time(NULL);
  char* szTest = NULL;
  char* searchText = NULL;
  asprintf(&searchText, "%s", search);

  int searchStart = 0, searchStop = 0;
  if (useTime)
  {
	searchStart = startTime;
	searchStop = stopTime;
	if (searchStop < searchStart)
		searchStop += 2400;
  }
  int minSearchDuration = 0;
  int maxSearchDuration = 0;
  if (useDuration)
  {
      minSearchDuration = minDuration;
      maxSearchDuration = maxDuration;
  }

  for (cEvent *p = p1; p; p = schedules->Events()->Next(p))
  {
     if(!p) 
     {
        break;
     }
     if (szTest)
     {
	free(szTest);
	szTest = NULL;
     }
     asprintf(&szTest, "%s%s%s", (useTitle?p->Title():""),
				 (useSubtitle?p->ShortText():""),
				 (useDescription?p->Description():""));     
	
     if (tNow < p->EndTime()) 
     {
        if (!useCase)
        {
		ToLower(szTest);
                ToLower(searchText);
        }
	
       	bool bTesting=false;
        if (mode == 0)
	{
	    if (!strstr(szTest, searchText)) 
		continue;
	    else
		bTesting = true;
	}
        else
		bTesting = false;
        if (mode > 0)
        { 
	    bTesting = false;
	    char *pstrSearchToken, *pptr;
	    bool bFirst=true;
	    char *pstrSearch=strdup(searchText);
	    pstrSearchToken=strtok_r(pstrSearch, " ,;|", &pptr);
	    while(pstrSearchToken) 
	    {
		if(szTest && strstr(szTest, pstrSearchToken)) 
		{ 
		    if(mode==1) 
		    { // means AND
			if(bFirst) 
			{
			    bTesting=true;
			    bFirst=false;
			} 
			else
			    bTesting&=true;
		    }
		    else
			bTesting|=true;
		} 
		else 
		{// not found!!
		    if(mode==1) 
		    { // means AND
			bTesting=false;
			bFirst=false;
		    }
		}
		pstrSearchToken=strtok_r(NULL, " ,;|", &pptr);
	    }
	    free(pstrSearch);
        }
        if(!bTesting)
	    continue;       

        if (useTime)
        {
		time_t tEvent = p->StartTime();
		struct tm tmEvent;
		localtime_r(&tEvent, &tmEvent);
		int eventStart = tmEvent.tm_hour*100 + tmEvent.tm_min;	
		int eventStart2 = tmEvent.tm_hour*100 + tmEvent.tm_min + 2400;	
		if ((eventStart < searchStart || eventStart > searchStop) &&
		    (eventStart2 < searchStart || eventStart2 > searchStop))
			continue;
        }
        if (useDuration)
        {
		int duration = p->Duration()/60;
		if (minSearchDuration > duration || maxSearchDuration < duration)
		    continue;
        }

	if (useDayOfWeek)
	{
	    time_t tEvent = p->StartTime();
	    struct tm tmEvent;
	    localtime_r(&tEvent, &tmEvent);
	    if (DayOfWeek >= 0 && DayOfWeek != tmEvent.tm_wday)
		continue;
	    if (DayOfWeek < 0)
	    {	      
		int iFound = 0;
		for(int i=0; i<7; i++)
		    if (abs(DayOfWeek) & (int)pow(2,i) && i == tmEvent.tm_wday)
		    {
			iFound = 1;
			break;
		    }
		if (!iFound)
		    continue;
	    }
	}

	pe=p;
	break;
     }
  }
  if (szTest)
	free(szTest);
  free(searchText);
  return pe;
}

// -- cSearchExts ----------------------------------------------------------------

cSearchExts::cSearchExts(void)
{
}


// --- cMenuEditSearchExt --------------------------------------------------------


cMenuEditSearchExt::cMenuEditSearchExt(cSearchExt *SearchExt, bool New)
:cOsdMenu(tr("Edit search"),30)
{
  asprintf(&SearchModes[0], tr("Phrase"));
  asprintf(&SearchModes[1], tr("all words"));
  asprintf(&SearchModes[2], tr("one word"));

  asprintf(&DaysOfWeek[0], WeekDayName(0));
  asprintf(&DaysOfWeek[1], WeekDayName(1));
  asprintf(&DaysOfWeek[2], WeekDayName(2));
  asprintf(&DaysOfWeek[3], WeekDayName(3));
  asprintf(&DaysOfWeek[4], WeekDayName(4));
  asprintf(&DaysOfWeek[5], WeekDayName(5));
  asprintf(&DaysOfWeek[6], WeekDayName(6));
  asprintf(&DaysOfWeek[7], tr("user defined"));

  searchExt = SearchExt;
  addIfConfirmed = New;
  if (searchExt) 
  {
     data = *searchExt;
     useTime = data.useTime;
     UserDefDayOfWeek = 0;
     if (searchExt->DayOfWeek < 0)
     {
	 UserDefDayOfWeek = searchExt->DayOfWeek;
	 data.DayOfWeek = 7;
     }
     if (data.useChannel)
     {
	 channelMin = data.channelMin->Number();
	 channelMax = data.channelMax->Number();
     }
     else
        channelMin = channelMax = cDevice::CurrentChannel();
     Set();
  }
}

void cMenuEditSearchExt::Set()
{
  int current = Current();
  Clear();
  Add(new cMenuEditStrItem( tr("Search term"), data.search, sizeof(data.search), tr(FileNameChars)));
  Add(new cMenuEditStraItem(tr("Search mode"),     &data.mode, 3, SearchModes));
  Add(new cMenuEditBoolItem( tr("Match case"), &data.useCase, tr("no"), tr("yes")));
  Add(new cMenuEditBoolItem( tr("Use Title"), &data.useTitle, tr("no"), tr("yes")));
  Add(new cMenuEditBoolItem( tr("Use Subtitle"), &data.useSubtitle, tr("no"), tr("yes")));
  Add(new cMenuEditBoolItem( tr("Use Description"), &data.useDescription, tr("no"), tr("yes")));
  Add(new cMenuEditBoolItem( tr("Use Channel"), &data.useChannel, tr("no"), tr("yes")));
  if (data.useChannel)
  {
	Add(new cMenuEditChanItem(tr("  from Channel"),      &channelMin));
	Add(new cMenuEditChanItem(tr("  to Channel"),      &channelMax));
  }
  Add(new cMenuEditBoolItem( tr("Use Time"), &useTime, tr("no"), tr("yes")));
  if (useTime == true)
  {
     Add(new cMenuEditTimeItem(tr("  Start after"),        &data.startTime));
     Add(new cMenuEditTimeItem(tr("  Start before"),         &data.stopTime));
  }
  Add(new cMenuEditBoolItem( tr("Use Duration"), &data.useDuration, tr("no"), tr("yes")));
  if (data.useDuration == true)
  {
     Add(new cMenuEditTimeItem(tr("  Min. Duration"), &data.minDuration));
     Add(new cMenuEditTimeItem(tr("  Max. Duration"), &data.maxDuration));
  }
  Add(new cMenuEditBoolItem( tr("Use Day of Week"), &data.useDayOfWeek, tr("no"), tr("yes")));
  if (data.useDayOfWeek)
      Add(new cMenuEditStraItem(tr("  Day of Week"),     &data.DayOfWeek, 8, DaysOfWeek));

  Add(new cMenuEditBoolItem( tr("Use as Search Timer"), &data.useAsSearchTimer, tr("no"), tr("yes")));
  if (data.useAsSearchTimer)
  {
      Add(new cMenuEditBoolItem( tr("  Serial recording"), &data.useEpisode, tr("no"), tr("yes")));
      Add(new cMenuEditStrItem( tr("  Directory"), data.directory, sizeof(data.directory), tr(FileNameChars)));
  }
  SetCurrent(Get(current));
}

cMenuEditSearchExt::~cMenuEditSearchExt()
{
  if (searchExt && addIfConfirmed)
     delete searchExt; // apparently it wasn't confirmed
}


eOSState cMenuEditSearchExt::ProcessKey(eKeys Key)
{
  int iTemp_useTime = useTime;
  int iTemp_useChannel = data.useChannel;
  int iTemp_useDuration = data.useDuration;
  int iTemp_useDayOfWeek = data.useDayOfWeek;
  int iTemp_useAsSearchTimer = data.useAsSearchTimer;

  eOSState state = cOsdMenu::ProcessKey(Key);

  if (useTime != iTemp_useTime ||
      iTemp_useChannel != data.useChannel ||
      iTemp_useDuration != data.useDuration ||
      iTemp_useDayOfWeek != data.useDayOfWeek ||
      iTemp_useAsSearchTimer != data.useAsSearchTimer)
  {
     Set();
     Display();
  }
  const char* ItemText = Get(Current())->Text();

  if (!HasSubMenu())
  {
      if (strstr(ItemText, tr("  from Channel")) == ItemText && channelMin > channelMax)
      {
	  channelMax = channelMin;
	  Set();
	  Display();
      }
      if (strstr(ItemText, tr("  to Channel")) == ItemText && channelMax < channelMin)
      {
	  channelMin = channelMax;
	  Set();
	  Display();
      }
  }

  int iOnUserDefDayItem = 0;
  if (!HasSubMenu() && strstr(ItemText, tr("  Day of Week")) == ItemText)
      if (data.DayOfWeek == 7)
      {
	  SetHelp(tr("Edit"));
	  iOnUserDefDayItem = 1;
      }
      else
	  SetHelp(NULL);

  if (state == osUnknown) {
    switch (Key) {
      case kOk:       if (!strlen(data.search)) {
                        Skins.Message(mtError, tr("Error$Missing search text!"));
                        break;
                      }
                      if (data.useChannel) {
                        cChannel *ch = Channels.GetByNumber(channelMin);
                        if (ch)
                          data.channelMin = ch;
                        else {
                          Skins.Message(mtError, tr("*** Invalid Channel ***"));
                          break;
                        }
                        ch = Channels.GetByNumber(channelMax);
                        if (ch)
                          data.channelMax = ch;
                        else {
                          Skins.Message(mtError, tr("*** Invalid Channel ***"));
                          break;
                        }

                      }
                      if ((int(strlen(data.search)) < 3) && !Interface->Confirm(tr("Edit$Search text to small - use anyway?")))
                        break;
                      data.useTime = useTime;

                     if (searchExt) 
		     {
			 *searchExt = data;
			 if (data.DayOfWeek == 7)
			     searchExt->DayOfWeek = UserDefDayOfWeek;
			 if (addIfConfirmed)
			     SearchExts.Add(searchExt);
			 SearchExts.Save();
			 addIfConfirmed = false;
		     }
                      return osBack;
	case kRed:
	    if (HasSubMenu())
		return osContinue;
	    if (iOnUserDefDayItem)
		state = AddSubMenu(new cMenuEditDaysOfWeek(&UserDefDayOfWeek));
	    break;

	case kBlue:   
	case kGreen:
	case kYellow: state = osContinue;
      default: break;
    }
  }

  return state;
}



// --- cMenuSearchExtItem ----------------------------------------------------------

class cMenuSearchExtItem : public cOsdItem {
  private:
public:
  cSearchExt* searchExt;
  cMenuSearchExtItem(cSearchExt* SearchExt);
  int Compare(const cListObject &ListObject) const;
  void Set(void);
  };

cMenuSearchExtItem::cMenuSearchExtItem(cSearchExt* SearchExt)
{
  searchExt = SearchExt;
  Set();
}

void cMenuSearchExtItem::Set(void)
{
  char *buffer = NULL;
  char *tmp[]  = { "", "", "", "", "" };

  if (searchExt->useAsSearchTimer)
      asprintf(&tmp[0], ">");

  asprintf(&tmp[1], "\t%s", searchExt->search);

  if (searchExt->useChannel && searchExt->channelMin != searchExt->channelMax)
      asprintf(&tmp[2], "\t%d - %d", searchExt->channelMin->Number(),searchExt->channelMax->Number());
  else
      asprintf(&tmp[2], "\t%.10s", (searchExt->useChannel?searchExt->channelMin->Name():""));

  if (searchExt->useTime)
  {
	  asprintf(&tmp[3], "\t%02d:%02d", searchExt->startTime / 100, searchExt->startTime % 100);
	  asprintf(&tmp[4], "\t%02d:%02d", searchExt->stopTime / 100, searchExt->stopTime % 100);
  }
  else
  {
	  asprintf(&tmp[3], "\t--:--");
	  asprintf(&tmp[4], "\t--:--");
  }
	
  asprintf(&buffer, "%s%s%s%s%s", tmp[0], tmp[1], tmp[2], tmp[3], tmp[4]);
  SetText(buffer, false);
}

int cMenuSearchExtItem::Compare(const cListObject &ListObject) const
{
  cMenuSearchExtItem *p = (cMenuSearchExtItem *)&ListObject;
  return strcasecmp(searchExt->search, p->searchExt->search);
}

// --- cMenuEPGSearchExt ----------------------------------------------------------
cMenuEPGSearchExt::cMenuEPGSearchExt()
:cOsdMenu(tr("Search"), 2, 20, 12, 6, 6)
{
  cSearchExt *SearchExt = SearchExts.First();
  while (SearchExt) {
    Add(new cMenuSearchExtItem(SearchExt));
    SearchExt = SearchExts.Next(SearchExt);
  }

  SetHelp(tr("Edit"), tr("New"), tr("Delete"), tr("Start"));
  Sort();
  Display();
}

cSearchExt *cMenuEPGSearchExt::CurrentSearchExt(void)
{
    cMenuSearchExtItem *item = (cMenuSearchExtItem *)Get(Current());
    return item ? item->searchExt : NULL;
}


eOSState cMenuEPGSearchExt::New(void)
{
  if (HasSubMenu())
    return osContinue;
  return AddSubMenu(new cMenuEditSearchExt(new cSearchExt, true));
}

eOSState cMenuEPGSearchExt::Delete(void)
{
  cSearchExt *curSearchExt = CurrentSearchExt();
  if (curSearchExt) {
    if (Interface->Confirm(tr("Edit$Delete Search?"))) {
      SearchExts.Del(curSearchExt);
      SearchExts.Save();
      cOsdMenu::Del(Current());
      Display();
    }
  }
  return osContinue;
}

eOSState cMenuEPGSearchExt::Search(void)
{
    return AddSubMenu(new cMenuSearchResults(CurrentSearchExt()));
}

eOSState cMenuEPGSearchExt::ProcessKey(eKeys Key)
{
  int SearchNumber = HasSubMenu() ? Count() : -1;
  eOSState state = cOsdMenu::ProcessKey(Key);
  if (state == osUnknown) {
    switch (Key) {
	case kOk:
	case kBlue:
	    if (CurrentSearchExt())
		state = Search();
	    else
		state = osContinue;
	    break;
	case kRed:
 	  if (HasSubMenu())
	    return osContinue;
	  if (CurrentSearchExt())
	      state = AddSubMenu(new cMenuEditSearchExt(CurrentSearchExt()));
	  else
	      state = osContinue;
	  break;
     	case kGreen: state = New(); break;
        case kYellow: state = Delete(); break;

      default: break;
    }
  }
  if (SearchNumber >= 0 && !HasSubMenu() && SearchExts.Get(SearchNumber)) {
     // a newly created search was confirmed with Ok
     Add(new cMenuSearchExtItem(SearchExts.Get(SearchNumber)));
     Display();
     }

  return state;
}

cMenuEditDaysOfWeek::cMenuEditDaysOfWeek(int* DaysOfWeek)
:cOsdMenu(tr("Edit user defined Days Of Week"),30)
{
    int i=0;
    pDaysOfWeek = DaysOfWeek;
    
    for(i=0; i<7; i++)
	Days[i]=(abs(*pDaysOfWeek) & (int)pow(2,i));
    
    for(i=0; i<7; i++)
	Add(new cMenuEditBoolItem( WeekDayName(i), &Days[i], tr("no"), tr("yes")));

    SetHelp(NULL);
}

eOSState cMenuEditDaysOfWeek::ProcessKey(eKeys Key)
{
    int i=0;
    eOSState state = cOsdMenu::ProcessKey(Key);
    if (state == osUnknown) {
	switch (Key) {
	    case kOk:
		*pDaysOfWeek = 0;
		for(i=0; i<7; i++)
		    *pDaysOfWeek += Days[i]?((int)pow(2,i)):0;
		*pDaysOfWeek = -*pDaysOfWeek;
		state = osBack;
	    default: break;
	}
    }
    return state;
}
