#include "epgsearchsetup.h"
#include "epgsearchcfg.h"
#include "searchtimer_thread.h"

const char *RedKeyMode[2];
const char *BlueKeyMode[2];
const char *ProgressbarMode[3];

cMenuEPGSearchSetup::cMenuEPGSearchSetup(void)
{
  RedKeyMode[0] = tr("Standard");
  RedKeyMode[1] = tr("Commands");

  BlueKeyMode[0] = tr("Standard");
  BlueKeyMode[1] = tr("Search");

  ProgressbarMode[0] = tr("no");
  ProgressbarMode[1] = tr("Pipes");
  ProgressbarMode[2] = tr("graphical");
  Setup();
}

void cMenuEPGSearchSetup::Setup(void)
{
  data = EPGSearchConfig;
  Set();
}

void cMenuEPGSearchSetup::Set()
{
  int current = Current();
  Clear();
  Add(new cMenuEditBoolItem(  tr("Hide Mainmenu Entry"),         &data.hidemenu,       tr("no"),      tr("yes")));
  Add(new cMenuEditStraItem(tr("Red key"), &data.redkeymode, 2, RedKeyMode));
  Add(new cMenuEditStraItem(tr("Blue key"), &data.bluekeymode, 2, BlueKeyMode));
  Add(new cMenuEditStraItem(tr("Show progress in 'Now'"),         &data.showProgress, 3, ProgressbarMode));
  if (data.showProgress)
      Add(new cMenuEditIntItem(tr("  Display Width"), &data.ProgressWidth, 4, 12));
  Add(new cMenuEditBoolItem(  tr("Show channel numbers"),         &data.showChannelNr,       tr("no"),      tr("yes")));

#if VDRVERSNUM && VDRVERSNUM >= 10315
  Add(new cMenuEditBoolItem(  tr("Use Short Channel Names"),         &data.useShortChNames,       tr("no"),      tr("yes")));
#endif

  Add(new cMenuEditBoolItem(  tr("Use Search Timers"),         &data.useSearchTimers,       tr("no"),      tr("yes")));
  if (data.useSearchTimers)
  {
      Add(new cMenuEditIntItem(tr("  Update Intervall [min]"), &data.UpdateIntervall, 1, 9999));
      Add(new cMenuEditIntItem(tr("  SVDRP Port"), &data.SVDRPPort, 1, 99999));
  }
  Add(new cMenuEditIntItem(tr("Time Intervall for FR/FF [min]"), &data.timeShiftValue, 1, 9999));
  Add(new cMenuEditBoolItem(  tr("Toggle Green/Yellow"),         &data.toggleGreenYellow,       tr("no"),      tr("yes")));
  char* szUseUserTime = NULL;
  asprintf(&szUseUserTime, "%s %d", tr("Use User Time"), 1);
  Add(new cMenuEditBoolItem(szUseUserTime, &data.ShowModes[showUserMode1].bUseIt,       tr("no"),      tr("yes")));
  if (data.ShowModes[showUserMode1].GetUsage())
  {
      Add(new cMenuEditStrItem(tr("  Description"), data.ShowModes[showUserMode1].szDescription, sizeof(data.ShowModes[showUserMode1].szDescription), tr(FileNameChars)));
      Add(new cMenuEditTimeItem(tr("  Time"), &data.ShowModes[showUserMode1].iTime));
  } 
  free(szUseUserTime);

  asprintf(&szUseUserTime, "%s %d", tr("Use User Time"), 2);
  Add(new cMenuEditBoolItem( szUseUserTime, &data.ShowModes[showUserMode2].bUseIt,       tr("no"),      tr("yes")));
  if (data.ShowModes[showUserMode2].GetUsage())
  {
      Add(new cMenuEditStrItem(tr("  Description"), data.ShowModes[showUserMode2].szDescription, sizeof(data.ShowModes[showUserMode2].szDescription), tr(FileNameChars)));
      Add(new cMenuEditTimeItem(tr("  Time"), &data.ShowModes[showUserMode2].iTime));
  } 
  free(szUseUserTime);

  asprintf(&szUseUserTime, "%s %d", tr("Use User Time"), 3);
  Add(new cMenuEditBoolItem(szUseUserTime, &data.ShowModes[showUserMode3].bUseIt,       tr("no"),      tr("yes")));
  if (data.ShowModes[showUserMode3].GetUsage())
  {
      Add(new cMenuEditStrItem(tr("  Description"), data.ShowModes[showUserMode3].szDescription, sizeof(data.ShowModes[showUserMode3].szDescription), tr(FileNameChars)));
      Add(new cMenuEditTimeItem(tr("  Time"), &data.ShowModes[showUserMode3].iTime));
  } 
  free(szUseUserTime);

  asprintf(&szUseUserTime, "%s %d", tr("Use User Time"), 4);
  Add(new cMenuEditBoolItem(szUseUserTime, &data.ShowModes[showUserMode4].bUseIt,       tr("no"),      tr("yes")));
  if (data.ShowModes[showUserMode4].GetUsage())
  {
      Add(new cMenuEditStrItem(tr("  Description"), data.ShowModes[showUserMode4].szDescription, sizeof(data.ShowModes[showUserMode4].szDescription), tr(FileNameChars)));
      Add(new cMenuEditTimeItem(tr("  Time"), &data.ShowModes[showUserMode4].iTime));
  } 
  free(szUseUserTime);

 SetCurrent(Get(current));
  Display();
}

void cMenuEPGSearchSetup::Store(void)
{
    bool bRestart = false;
    if (EPGSearchConfig.useSearchTimers != data.useSearchTimers)
    {
	bRestart = true;
	cSearchTimerThread::Exit();
    }

    EPGSearchConfig = data;

    SetupStore("HideMenu",  EPGSearchConfig.hidemenu);
    SetupStore("RedKeyMode",  EPGSearchConfig.redkeymode);
    SetupStore("BlueKeyMode",  EPGSearchConfig.bluekeymode);
    SetupStore("ShowProgress",  EPGSearchConfig.showProgress);
    SetupStore("ProgressWidth",  EPGSearchConfig.ProgressWidth);
    SetupStore("ShowChannelNr",  EPGSearchConfig.showChannelNr);
    
    SetupStore("UserMode1UseIt",  EPGSearchConfig.ShowModes[showUserMode1].GetUsage());
    SetupStore("UserMode1Description",  EPGSearchConfig.ShowModes[showUserMode1].GetDescription());
    SetupStore("UserMode1Time",  EPGSearchConfig.ShowModes[showUserMode1].GetTime());
    
    SetupStore("UserMode2UseIt",  EPGSearchConfig.ShowModes[showUserMode2].GetUsage());
    SetupStore("UserMode2Description",  EPGSearchConfig.ShowModes[showUserMode2].GetDescription());
    SetupStore("UserMode2Time",  EPGSearchConfig.ShowModes[showUserMode2].GetTime());
  
    SetupStore("UserMode3UseIt",  EPGSearchConfig.ShowModes[showUserMode3].GetUsage());
    SetupStore("UserMode3Description",  EPGSearchConfig.ShowModes[showUserMode3].GetDescription());
    SetupStore("UserMode3Time",  EPGSearchConfig.ShowModes[showUserMode3].GetTime());
    
    SetupStore("UserMode4UseIt",  EPGSearchConfig.ShowModes[showUserMode4].GetUsage());
    SetupStore("UserMode4Description",  EPGSearchConfig.ShowModes[showUserMode4].GetDescription());
    SetupStore("UserMode4Time",  EPGSearchConfig.ShowModes[showUserMode4].GetTime());
    
    SetupStore("UseSearchTimers",  EPGSearchConfig.useSearchTimers);
    SetupStore("UpdateIntervall",  EPGSearchConfig.UpdateIntervall);
    SetupStore("SVDRPPort",  EPGSearchConfig.SVDRPPort);

    SetupStore("TimeIntervallFRFF",  EPGSearchConfig.timeShiftValue);
    SetupStore("ToggleGreenYellow",  EPGSearchConfig.toggleGreenYellow);

    SetupStore("ShortChannelNames",  EPGSearchConfig.useShortChNames);

    if (bRestart)
	cSearchTimerThread::Init();
}

eOSState cMenuEPGSearchSetup::ProcessKey(eKeys Key)
{
    int iTemp_useUserTime1 = data.ShowModes[showUserMode1].GetUsage();
    int iTemp_useUserTime2 = data.ShowModes[showUserMode2].GetUsage();
    int iTemp_useUserTime3 = data.ShowModes[showUserMode3].GetUsage();
    int iTemp_useUserTime4 = data.ShowModes[showUserMode4].GetUsage();
    int iTemp_showProgress = data.showProgress;
    int iTemp_useSearchTimers = data.useSearchTimers;

    eOSState state = cMenuSetupPage::ProcessKey(Key);

    if (iTemp_useUserTime1 != data.ShowModes[showUserMode1].GetUsage() ||
	iTemp_useUserTime2 != data.ShowModes[showUserMode2].GetUsage() ||
	iTemp_useUserTime3 != data.ShowModes[showUserMode3].GetUsage() ||
	iTemp_useUserTime4 != data.ShowModes[showUserMode4].GetUsage() ||
	iTemp_showProgress != data.showProgress ||
	iTemp_useSearchTimers != data.useSearchTimers)
    {
	Set();
	Display();
    }
    return state;
}
